DROP TABLE IF EXISTS `geni_users`;
CREATE TABLE `geni_users` (
  `urn` varchar(256) NOT NULL default '',
  `hrn` varchar(256) NOT NULL default '',
  `uid` varchar(8) NOT NULL default '',
  `idx` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `locked` datetime default NULL,
  `archived` datetime default NULL,
  `status` enum('active','archived','frozen') NOT NULL default 'frozen',
  `name` tinytext,
  `email` tinytext,
  `sa` varchar(256) NOT NULL default '',
  PRIMARY KEY  (`idx`),
  UNIQUE KEY `urn` (`urn`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_components`;
CREATE TABLE `geni_components` (
  `urn` varchar(256) NOT NULL default '',
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `url` tinytext,
  PRIMARY KEY  (`urn`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_authorities`;
CREATE TABLE `geni_authorities` (
  `hrn` varchar(256) NOT NULL default '',
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `type` enum('sa','ma','ch','cm','ses') NOT NULL default 'sa',
  `url` tinytext,
  `urn` varchar(256),
  PRIMARY KEY  (`urn`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_slices`;
CREATE TABLE `geni_slices` (
  `urn` varchar(256) NOT NULL default '',
  `idx` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `shutdown` datetime default NULL,
  `locked` datetime default NULL,
  `name` tinytext,
  `sa` varchar(256) NOT NULL default '',
  PRIMARY KEY  (`idx`),
  UNIQUE KEY `urn` (`urn`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_slivers`;
CREATE TABLE `geni_slivers` (
  `idx` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `hrn` varchar(256) NOT NULL default '',
  `nickname` varchar(256) default NULL,
  `slice` varchar(256) NOT NULL default '',
  `creator` varchar(256) NOT NULL default '',
  `resource_uuid` varchar(256) NOT NULL default '',
  `resource_type` varchar(40) NOT NULL default '',
  `resource_id` varchar(64) NOT NULL default '',
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `locked` datetime default NULL,
  `credential_idx` int(10) unsigned default NULL,
  `component_uuid` varchar(40) default NULL,
  `aggregate` varchar(256) default NULL,
  `status` varchar(16) NOT NULL default 'created',
  `state` varchar(16) NOT NULL default 'stopped',
  `rspec_string` text,
  PRIMARY KEY  (`idx`),
  INDEX `slice` (`slice`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_aggregates`;
CREATE TABLE `geni_aggregates` (
  `idx` mediumint(8) unsigned NOT NULL AUTO_INCREMENT,
  `hrn` varchar(256) NOT NULL default '',
  `nickname` varchar(256) default NULL,
  `type` varchar(40) NOT NULL default '',
  `slice` varchar(256) NOT NULL default '',
  `creator` varchar(256) NOT NULL default '',
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `locked` datetime default NULL,
  `registered` datetime default NULL,
  `credential_idx` int(10) unsigned default NULL,
  `component_idx` int(10) unsigned NOT NULL default '0',
  `aggregate_idx` int(10) unsigned default NULL,
  `status` varchar(16) NOT NULL default 'created',
  `state` varchar(16) NOT NULL default 'stopped',
  PRIMARY KEY  (`idx`),
  INDEX `slice` (`slice`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_credentials`;
CREATE TABLE `geni_credentials` (
  `idx` mediumint(8) unsigned NOT NULL default '0',
  `uuid` varchar(40) NOT NULL default '',
  `owner` varchar(256) NOT NULL default '',
  `this` varchar(256) NOT NULL default '',
  `created` datetime default NULL,
  `valid_until` datetime default NULL,
  `credential_string` text,
  PRIMARY KEY  (`idx`),
  INDEX `owner` (`owner`),
  INDEX `this` (`this`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_certificates`;
CREATE TABLE `geni_certificates` (
  `urn` varchar(256) NOT NULL default '',
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `revoked` datetime default NULL,
  `serial` int(10) unsigned NOT NULL default '0',
  `cert` text,
  `DN` text,
  `privkey` text,
  `uri` text,
  PRIMARY KEY  (`urn`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

DROP TABLE IF EXISTS `geni_crls`;
CREATE TABLE `geni_crls` (
  `uuid` varchar(40) NOT NULL default '',
  `created` datetime default NULL,
  `expires` datetime default NULL,
  `cert` text,
  `DN` text,
  PRIMARY KEY  (`uuid`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;

CREATE TABLE `geni_manifests` (
  `idx` int(10) unsigned NOT NULL auto_increment,
  `slice` varchar(256) NOT NULL default '',
  `created` datetime default NULL,
  `manifest` text,
  PRIMARY KEY  (`idx`),
  UNIQUE KEY `slice` (`slice`)
) ENGINE=MyISAM DEFAULT CHARSET=latin1;
